#!/usr/bin/python
"""
Copyright (c) 2015 SONATA-NFV and Paderborn University
ALL RIGHTS RESERVED.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Neither the name of the SONATA-NFV [, ANY ADDITIONAL AFFILIATION]
nor the names of its contributors may be used to endorse or promote
products derived from this software without specific prior written
permission.

This work has been performed in the framework of the SONATA project,
funded by the European Commission under Grant number 671517 through
the Horizon 2020 and 5G-PPP programmes. The authors would like to
acknowledge the contributions of their colleagues of the SONATA
partner consortium (www.sonata-nfv.eu).
"""
"""
 Simple CLI client to interact with a running emulator.

 The CLI offers different tools, e.g., compute, network, ...
 Each of these tools is implemented as an independent Python
 module.

 cli compute start dc1 my_name flavor_a
 cli network create dc1 11.0.0.0/24
"""

import sys

from emuvim.cli import compute
from emuvim.cli import datacenter
from emuvim.cli import monitor
from emuvim.cli import network
from emuvim.cli.rest import compute as restcom
from emuvim.cli.rest import datacenter as restdc
from emuvim.cli.rest import monitor as restmon
from emuvim.cli.rest import network as restnetw


def main():
    if len(sys.argv) < 2:
        print("Usage: son-emu-cli <toolname> <arguments>")
        exit(0)
    if sys.argv[1] == "compute-zapi":
        compute.main(sys.argv[2:])
    elif sys.argv[1] == "network-zapi":
        network.main(sys.argv[2:])
    elif sys.argv[1] == "datacenter-zapi":
        datacenter.main(sys.argv[2:])
    elif sys.argv[1] == "monitor-zapi":
        monitor.main(sys.argv[2:])
    elif sys.argv[1] == "monitor":
        restmon.main(sys.argv[2:])
    elif sys.argv[1] == "network":
        restnetw.main(sys.argv[2:])
    elif sys.argv[1] == "compute":
        restcom.main(sys.argv[2:])
    elif sys.argv[1] == "datacenter":
        restdc.main(sys.argv[2:])


if __name__ == '__main__':
    main()
